@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,28 +20,29 @@
 #ifndef LIBRARY_H
 #define LIBRARY_H
 
-#define MAX_ARTIST 128
-#define MAX_TITLE 128
-#define MAX_NAME 512
-#define MAX_PATHNAME 1024
+#include <stdbool.h>
 
-struct record_t {
-    char artist[MAX_ARTIST],
-        title[MAX_TITLE],
-        name[MAX_NAME],
-        pathname[MAX_PATHNAME];
+#include "listing.h"
+
+#define CRATE_ALL "All records"
+
+struct crate_t {
+    bool is_fixed;
+    char *name;
+    struct listing_t listing;
 };
 
-/* Library acts as a storage of the actual strings */
-
 struct library_t {
-    struct record_t *record;
-    int size, entries;
+    struct listing_t storage;
+    struct crate_t **crate;
+    int crates;
 };
 
 int library_init(struct library_t *li);
 void library_clear(struct library_t *li);
-int library_add(struct library_t *li, struct record_t *lr);
-int library_import(struct library_t *li, char *path);
+struct crate_t* library_new_crate(struct library_t *lib, char *name,
+                                  bool is_fixed);
+struct crate_t* library_get_crate(struct library_t *lib, char *name);
+int library_import(struct library_t *lib, const char *scan, const char *path);
 
 #endif
