@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,21 +20,24 @@
 #ifndef LISTING_H
 #define LISTING_H
 
-#include "library.h"
+struct record_t {
+    char *pathname, *artist, *title;
+};
 
-/* Listing points to records within a library */
+/* Listing points to records, but does not manage those pointers */
 
 struct listing_t {
     struct record_t **record;
     int size, entries;
 };
 
-int listing_init(struct listing_t *ls);
+void listing_init(struct listing_t *ls);
 void listing_clear(struct listing_t *ls);
 void listing_blank(struct listing_t *ls);
 int listing_add(struct listing_t *li, struct record_t *lr);
-int listing_add_library(struct listing_t *li, struct library_t *lb);
-int listing_match(struct listing_t *src, struct listing_t *dest, char *match);
+int listing_copy(const struct listing_t *src, struct listing_t *dest);
+int listing_match(struct listing_t *src, struct listing_t *dest,
+		  const char *match);
 void listing_debug(struct listing_t *ls);
 void listing_sort(struct listing_t *ls);
 
