@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,6 +17,8 @@
  *
  */
 
+#define _BSD_SOURCE /* vfork() */
+#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -46,38 +48,53 @@ static int start_import(struct track_t *tr, const char *path)
         perror("pipe");
         return -1;
     }
-    
+    if(fcntl(pstdout[0], F_SETFL, O_NONBLOCK) == -1) {
+        perror("fcntl");
+        return -1;
+    }
+
     tr->pid = vfork();
     
-    if(tr->pid < 0) {
+    if(tr->pid == -1) {
         perror("vfork");
         return -1;
         
     } else if(tr->pid == 0) { /* child */
 
-        close(pstdout[0]);
-        dup2(pstdout[1], STDOUT_FILENO);
-        close(pstdout[1]);
-        
+        /* Reconnect stdout to this process, leave stderr to terminal */
+
+        if(close(pstdout[0]) != 0) {
+            perror("close");
+            abort();
+        }
+        if(dup2(pstdout[1], STDOUT_FILENO) == -1) {
+            perror("dup2");
+            _exit(EXIT_FAILURE); /* vfork() was used */
+        }
+        if(close(pstdout[1]) != 0) {
+            perror("close");
+            abort();
+        }
+
         if(execl(tr->importer, "import", path, NULL) == -1) {
             perror("execl");
-            exit(-1);
-            return 0;
+            fprintf(stderr, "Failed to launch importer %s\n", tr->importer);
+            _exit(EXIT_FAILURE); /* vfork() was used */
         }
+
+        abort(); /* execl() never returns */
+    }
+
+    if(close(pstdout[1]) != 0) {
+        perror("close");
+        abort();
     }
 
-    close(pstdout[1]);
     tr->fd = pstdout[0];
-    if(fcntl(tr->fd, F_SETFL, O_NONBLOCK) == -1) {
-        perror("fcntl");
-        return -1;
-    }
-
     tr->bytes = 0;
     tr->length = 0;
     tr->ppm = 0;
     tr->overview = 0;
-    tr->eof = 0;
     tr->rate = TRACK_RATE;
 
     return 0;
@@ -87,22 +104,41 @@ static int start_import(struct track_t *tr, const char *path)
 /* Conclude the importer process. To be called whether the importer
  * was aborted or completed successfully */
 
-static int stop_import(struct track_t *tr)
+static void stop_import(struct track_t *tr)
 {
     int status;
 
-    if(close(tr->fd) == -1) {
+    assert(tr->pid != 0);
+
+    if(close(tr->fd) != 0) {
         perror("close");
-        return -1;
+        abort();
     }
 
     if(waitpid(tr->pid, &status, 0) == -1) {
         perror("waitpid");
-        return -1;
+        abort();
     }
-    fprintf(stderr, "Track importer exited with status %d.\n", status);
+    tr->pid = 0;
 
-    return 0;
+    if(WIFEXITED(status) && WEXITSTATUS(status) == EXIT_SUCCESS) {
+        fputs("Track import completed.\n", stderr);
+    } else {
+        fputs("Track import did not complete successfully.\n", stderr);
+    }
+}
+
+
+/* Prematurely abort the import process */
+
+static void abort_import(struct track_t *tr)
+{
+    if(kill(tr->pid, SIGTERM) == -1) {
+        perror("kill");
+        abort();
+    }
+
+    stop_import(tr);
 }
 
 
@@ -171,7 +207,7 @@ static int read_from_pipe(struct track_t *tr)
         ls = s % TRACK_BLOCK_SAMPLES;
         
         v = (abs(block->pcm[ls * TRACK_CHANNELS])
-             + abs(block->pcm[ls * TRACK_CHANNELS]));
+             + abs(block->pcm[ls * TRACK_CHANNELS + 1]));
 
         /* PPM-style fast meter approximation */
 
@@ -208,47 +244,33 @@ void track_init(struct track_t *tr, const char *importer)
 
     tr->artist = NULL;
     tr->title = NULL;
-    tr->name = NULL;
 
     tr->blocks = 0;
     tr->bytes = 0;
     tr->length = 0;
     tr->rate = TRACK_RATE;
 
-    pthread_mutex_init(&tr->mx, 0); /* always returns zero */
-
-    tr->status = TRACK_STATUS_VALID;
+    if (pthread_mutex_init(&tr->mx, 0) != 0)
+        abort();
 }
 
 
 /* Destroy this track from memory, and any child process */
 
-int track_clear(struct track_t *tr)
+void track_clear(struct track_t *tr)
 {
     int n;
 
     /* Force a cleanup of whichever state we are in */
 
-    if(tr->status > TRACK_STATUS_VALID) {
-        if(tr->status == TRACK_STATUS_IMPORTING) {
-            if(kill(tr->pid, SIGKILL) == -1) {
-                perror("kill");
-                return -1;
-            }
-        }
-        if(stop_import(tr) == -1)
-            return -1;
-    }
+    if(tr->pid != 0)
+        abort_import(tr);
 
     for(n = 0; n < tr->blocks; n++)
         free(tr->block[n]);
 
-    if(pthread_mutex_destroy(&tr->mx) == EBUSY) {
-        fprintf(stderr, "Track busy on destroy.\n");
-        return -1;
-    }
-
-    return 0;
+    if(pthread_mutex_destroy(&tr->mx) != 0)
+        abort();
 }
 
 
@@ -261,7 +283,7 @@ int track_pollfd(struct track_t *tr, struct pollfd *pe)
 
     LOCK(tr);
 
-    if(tr->status == TRACK_STATUS_IMPORTING && !tr->eof) {
+    if(tr->pid != 0) {
         pe->fd = tr->fd;
         pe->revents = 0;
         pe->events = POLLIN | POLLHUP | POLLERR;
@@ -291,10 +313,10 @@ int track_handle(struct track_t *tr)
 
     LOCK(tr);
 
-    if(tr->status == TRACK_STATUS_IMPORTING && !tr->eof) {
+    if(tr->pid != 0) {
         r = read_from_pipe(tr);
         if(r != 0)
-            tr->eof = 1;
+            stop_import(tr);
     }
 
     UNLOCK(tr);
@@ -313,19 +335,8 @@ int track_import(struct track_t *tr, const char *path)
 
     /* Abort any running import process */
 
-    if(tr->status != TRACK_STATUS_VALID) {
-        if(tr->status == TRACK_STATUS_IMPORTING) {
-            if(kill(tr->pid, SIGTERM) == -1) {
-                perror("kill");
-                UNLOCK(tr);
-                return -1;
-            }
-        }
-        if(stop_import(tr) == -1) {
-            UNLOCK(tr);
-            return -1;
-        }
-    }
+    if(tr->pid != 0)
+        abort_import(tr);
 
     /* Start the new import process */
 
@@ -334,7 +345,6 @@ int track_import(struct track_t *tr, const char *path)
         UNLOCK(tr);
         return -1;
     }
-    tr->status = TRACK_STATUS_IMPORTING;
 
     UNLOCK(tr);
 
@@ -342,19 +352,3 @@ int track_import(struct track_t *tr, const char *path)
 
     return 0;
 }
-
-
-/* Do any waiting for completion of importer process if needed */
-
-int track_wait(struct track_t *tr)
-{
-    LOCK(tr);
-
-    if(tr->status == TRACK_STATUS_IMPORTING && tr->eof) {
-        stop_import(tr);
-        tr->status = TRACK_STATUS_VALID;
-    }
-
-    UNLOCK(tr);
-    return 0;
-}
