@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,6 +21,7 @@
 #define TRACK_H
 
 #include <pthread.h>
+#include <stdbool.h>
 #include <sys/poll.h>
 #include <sys/types.h>
 
@@ -29,10 +30,6 @@
 #define TRACK_CHANNELS 2
 #define TRACK_RATE 44100
 
-#define TRACK_STATUS_UNKNOWN -1
-#define TRACK_STATUS_VALID 0
-#define TRACK_STATUS_IMPORTING 1
-
 #define TRACK_MAX_BLOCKS 64
 #define TRACK_BLOCK_SAMPLES (2048 * 1024)
 #define TRACK_PPM_RES 64
@@ -45,8 +42,8 @@ struct track_block_t {
 };
 
 struct track_t {
-    int status, fd, eof, rate;
-    pid_t pid;
+    int fd, rate;
+    pid_t pid; /* 0 if not importing */
     struct pollfd *pe;
     pthread_mutex_t mx;
     struct rig_t *rig;
@@ -54,7 +51,7 @@ struct track_t {
     /* pointers to external data */
    
     const char *importer, /* path to import script */
-        *artist, *title, *name;
+        *artist, *title;
     
     size_t bytes; /* loaded in */
     int length, /* track length in samples */
@@ -68,28 +65,46 @@ struct track_t {
 };
 
 void track_init(struct track_t *tr, const char *importer);
-int track_clear(struct track_t *tr);
+void track_clear(struct track_t *tr);
 int track_import(struct track_t *tr, const char *path);
 int track_pollfd(struct track_t *tr, struct pollfd *pe);
 int track_handle(struct track_t *tr);
 int track_abort(struct track_t *tr);
 int track_wait(struct track_t *tr);
 
-/* Macro functions, to force the code inline */
+/* Return true if the track importer is running, otherwise false */
 
-#define track_get_ppm(tr, s) \
-    ((tr)->block[(s) / TRACK_BLOCK_SAMPLES] \
-     ->ppm[((s) % TRACK_BLOCK_SAMPLES) / TRACK_PPM_RES])
+static inline bool track_is_importing(struct track_t *tr)
+{
+    return (tr->pid != 0);
+}
 
-#define track_get_overview(tr, s) \
-    ((tr)->block[(s) / TRACK_BLOCK_SAMPLES] \
-     ->overview[((s) % TRACK_BLOCK_SAMPLES) / TRACK_OVERVIEW_RES])
+/* Return the pseudo-PPM meter value for the given sample */
+
+static inline unsigned char track_get_ppm(struct track_t *tr, int s)
+{
+    struct track_block_t *b;
+    b = tr->block[s / TRACK_BLOCK_SAMPLES];
+    return b->ppm[(s % TRACK_BLOCK_SAMPLES) / TRACK_PPM_RES];
+}
+
+/* Return the overview meter value for the given sample */
+
+static inline unsigned char track_get_overview(struct track_t *tr, int s)
+{
+    struct track_block_t *b;
+    b = tr->block[s / TRACK_BLOCK_SAMPLES];
+    return b->overview[(s % TRACK_BLOCK_SAMPLES) / TRACK_OVERVIEW_RES];
+}
 
 /* Return a pointer to (not value of) the sample data for each channel */
 
-#define track_get_sample(tr, s) \
-    (&(tr)->block[(s) / TRACK_BLOCK_SAMPLES]->pcm[((s) % TRACK_BLOCK_SAMPLES) \
-                                                  * TRACK_CHANNELS])
+static inline signed short* track_get_sample(struct track_t *tr, int s)
+{
+    struct track_block_t *b;
+    b = tr->block[s / TRACK_BLOCK_SAMPLES];
+    return &b->pcm[(s % TRACK_BLOCK_SAMPLES) * TRACK_CHANNELS];
+}
 
 #endif
 
